/*
 *	CommandIR Firmware Updater
 *  (C) 2010-2011 InnovationOne Inc.
 *	v0: March 2010 - built on libusb-1
 *  v1: Oct 2010 - backported to LIRC's libusb-0.4
 *  v2: Apr 2011 - support upgrading only specific CommandIRs (for multi-device support)
 */

#include <errno.h>
#include <signal.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <usb.h>

#define USB_CMDIR_VENDOR_ID		0x10c4
#define USB_CMDIR_PRODUCT_ID	0x0003
#define USB_CMDIR_MINOR_BASE	192
#define MAX_FIRMWARE_PACKET 64
#define USB_TIMEOUT_MS 5000


static int upload_firmware(usb_dev_handle *cmdir_udev, int currentFirmwareRevision, char *firmware_name)
{
	// Let's read the whole thing into memory, then TX via USB with checksum
  unsigned char * buffer;
  FILE * firmware_file;
  int lSize, checkResult, transmittedBytes = 0, txThisTime, packetIdNumber = 0;
  unsigned char txPacket[513];	// extra byte just in case
  int x, send_status = 0, i;
  unsigned char checksum_count = 0;
  int packetsToExpect = 0;
  int transfered = 0;
  int bufRead = 0;
/*
  checkResult = system("md5sum -c checksum --status");

  if(checkResult)
  {
  	printf("MD5 checksum failed, refusing to update CommandIR with potentially bad firmware\n");
  	return;
  }
*/
  firmware_file = fopen(firmware_name,"r");	// CHECK IF THE FILE EXISTS
  if(!firmware_file)
  {
  	printf("Firmware file not found!\n");
  	return;
  }

  fseek (firmware_file , 0 , SEEK_END);
  lSize = ftell (firmware_file);
  rewind (firmware_file);

  buffer = malloc(lSize+1);

  while(!feof(firmware_file))
  {
  	bufRead+=fread(&buffer[bufRead], 1, lSize, firmware_file);		// hey, if we can read the whole file at once, great
  }
  fclose(firmware_file);

//	printf("Successfully read firmware file, length %d.\n", lSize);
	txPacket[0] = 1;
	txPacket[1] = 0;

	while(transmittedBytes < lSize)
	{
		txThisTime = lSize - transmittedBytes;
		if(txThisTime > (MAX_FIRMWARE_PACKET - 4) )
			txThisTime = (MAX_FIRMWARE_PACKET - 4);

		for(x = transmittedBytes; x < (transmittedBytes + txThisTime); x++)
		{
			checksum_count += buffer[x];
		}

		txPacket[2] = (unsigned char)(checksum_count);
		txPacket[3] = (unsigned char)txThisTime;
		memcpy(&txPacket[4], &buffer[transmittedBytes], txThisTime);
		transmittedBytes += txThisTime;

		// last packet?
		if(transmittedBytes == lSize)
		{
			txPacket[1] = 1;
		}

		printf("\rDelivering firmware; %d bytes complete", transfered);

		send_status=usb_bulk_write(
										cmdir_udev,
										1,
										txPacket,
										txThisTime + 4,
										USB_TIMEOUT_MS);

		if (send_status < 0) {
			fprintf(stderr, "Send error %d\n", send_status);
			return send_status;
		} else {
			transfered += send_status;
//			printf("Sent %d bytes ...\n", transfered);
		}

		txPacket[0] = 0;
		checksum_count = 0;
		if(send_status < 0)
		{
			printf("USB Write error sending firmware, aborting upgrade (Error %d).\n", send_status);
			break;
		}

	}

  printf("\nDone. Wait 5 seconds for the CommandIR to write flash and reboot.\n");

	return 0;

}



int main(int argc, char **argv)
{

	struct usb_bus *bus = 0;
	struct usb_device *dev = 0;
	usb_dev_handle *cmdir_udev;
	int busid, devid, thisbusid;
	
	printf("CommandIR Firmware Updater v2.0 (April 2011)\nfor CommandIR III Mini and Expandable.\n\n");
	
	if(argc < 5)
	{
	  printf("Calling commandir_firmware_updater directly is dangerous!  \nUse upgrade_commandir_firmware.sh instead to avoid loading an incorrect firmware.\n");
//		printf("Usage: %s bus_id device_id commandir_firmware_file.hex\n", argv[0]);
		return -1;
	}
	
	if(*argv[4] != 'Z')
	{
	  printf("Calling commandir_firmware_updater directly is dangerous!  \nUse upgrade_commandir_firmware.sh instead to avoid loading an incorrect firmware.\n");
	  exit(1);
	}
	
	busid = atoi(argv[1]);
	devid = atoi(argv[2]);
	
	usb_init();
	usb_find_busses();
	
	if(!usb_find_devices()) {
		return;
	}
	
	
	// Main Scan:
	for (bus = usb_busses; bus; bus = bus->next)
	{
	  thisbusid = bus->dirname[2] - '0' + (bus->dirname[1]-'0') * 
			10 + (bus->dirname[0]-'0') * 100;
	  if(busid != thisbusid) 
	    continue;
		for (dev = bus->devices; dev; dev = dev->next)
		{
			if ( (dev->descriptor.idVendor == USB_CMDIR_VENDOR_ID) && 
				(dev->descriptor.idProduct == 3) )
			{
			  /* Check the bus number and device number: */
			  if(dev->devnum != devid) 
			    continue;
			  printf("Upgrading CommandIR at bus:device %d : %d\n", thisbusid, dev->devnum);
				cmdir_udev = usb_open(dev);
				int r = 0;
	
				r = usb_claim_interface(cmdir_udev, 0);
				if(r < 0)
				{
					printf("Unable to claim CommandIR!\n");
					usb_close(cmdir_udev);
					continue;
				}
				
				
				// Does this CommandIR need to be upgraded?  Is iSerial < firmware's iSerial?
				
				// Upgrade firmware on this CommandIR:
				upload_firmware(cmdir_udev, dev->descriptor.iSerialNumber, argv[3]);

				usb_close(cmdir_udev);
				
			}// if it's a CommandIR
		}// for bus dev's
	}// for bus's
	
	
	return 0;
}


